//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Data/DataAccessWidget.cpp
//! @brief     Defines class DataAccessWidget
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Data/DataAccessWidget.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Data/Data1DItem.h"
#include "GUI/Model/Data/Data2DItem.h"

DataAccessWidget::DataAccessWidget(QWidget* parent)
    : QWidget(parent)
    , m_item(nullptr)
    , m_fftItem(nullptr)
{
}
DataAccessWidget::~DataAccessWidget() = default;

void DataAccessWidget::setJobOrRealItem(QObject* item)
{
    m_item = item;
    ASSERT(jobItem() || realItem());

    disconnect(m_item, &QObject::destroyed, this, nullptr);
    connect(m_item, &QObject::destroyed, this, [this]() { m_item = nullptr; });
}

void DataAccessWidget::setFourierItem(Data2DItem* fftItem)
{
    m_fftItem = fftItem;
}

QList<QAction*> DataAccessWidget::actionList()
{
    return QList<QAction*>();
}

JobItem* DataAccessWidget::jobItem() const
{
    return dynamic_cast<JobItem*>(m_item);
}

RealItem* DataAccessWidget::realItem() const
{
    return dynamic_cast<RealItem*>(m_item);
}

QObject* DataAccessWidget::jobRealBase() const
{
    return m_item;
}

Data1DItem* DataAccessWidget::realData1DItem() const
{
    return realDataItem<Data1DItem>();
}

Data1DItem* DataAccessWidget::simuData1DItem() const
{
    return simuDataItem<Data1DItem>();
}

Data1DItem* DataAccessWidget::diffData1DItem() const
{
    return diffDataItem<Data1DItem>();
}

Data2DItem* DataAccessWidget::realData2DItem() const
{
    return realDataItem<Data2DItem>();
}

Data2DItem* DataAccessWidget::simuData2DItem() const
{
    return simuDataItem<Data2DItem>();
}

Data2DItem* DataAccessWidget::diffData2DItem() const
{
    return diffDataItem<Data2DItem>();
}

QList<Data1DItem*> DataAccessWidget::mainData1DItems() const
{
    return mainDataItems<Data1DItem>();
}

QList<Data1DItem*> DataAccessWidget::allData1DItems() const
{
    return allDataItems<Data1DItem>();
}

Data1DItem* DataAccessWidget::currentData1DItem() const
{
    if (allData1DItems().size() == 0)
        return nullptr;
    return allData1DItems().first();
}

QList<Data2DItem*> DataAccessWidget::mainData2DItems() const
{
    if (m_fftItem)
        return {m_fftItem};
    return mainDataItems<Data2DItem>();
}

QList<Data2DItem*> DataAccessWidget::allData2DItems() const
{
    if (m_fftItem)
        return {m_fftItem};
    return allDataItems<Data2DItem>();
}

Data2DItem* DataAccessWidget::currentData2DItem() const
{
    if (allData2DItems().size() == 0)
        return nullptr;
    return allData2DItems().first();
}
